from decimal import Decimal


BONE = Decimal('1')
MIN_FEE = Decimal('0.000001')
MAX_FEE = Decimal('0.1')
INIT_POOL_SUPPLY = BONE * Decimal('100')
MIN_BOUND_TOKENS = 2
MAX_BOUND_TOKENS = 8
AMPLIFICATION_PARAMETER = Decimal('200')
MIN_WEIGHT = 0.01
_MAX_WEIGHTED_TOKENS = 100
_MAX_IN_RATIO = 0.3
_MAX_OUT_RATIO = 0.3
_MAX_INVARIANT_RATIO = 3
_MIN_INVARIANT_RATIO = 0.7


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['balpy',
 'balpy.balancerv2cad.scripts',
 'balpy.balancerv2cad.src.balancerv2cad',
 'balpy.balancerv2cad.src.balancerv2cad.logger',
 'balpy.balancerv2cad.tests',
 'balpy.balancerv2cad.tests.unit-test',
 'balpy.enums',
 'balpy.graph']

package_data = \
{'': ['*'],
 'balpy': ['abi/*',
           'balancerv2cad/*',
           'balancerv2cad/notebooks/*',
           'deployments/20210418-authorizer/abi/*',
           'deployments/20210418-authorizer/output/*',
           'deployments/20210418-vault/abi/*',
           'deployments/20210418-vault/output/*',
           'deployments/20210418-weighted-pool/abi/*',
           'deployments/20210418-weighted-pool/output/*',
           'deployments/20210624-stable-pool/abi/*',
           'deployments/20210624-stable-pool/output/*',
           'deployments/20210721-liquidity-bootstrapping-pool/abi/*',
           'deployments/20210721-liquidity-bootstrapping-pool/output/*',
           'deployments/20210727-meta-stable-pool/abi/*',
           'deployments/20210727-meta-stable-pool/output/*',
           'deployments/20210811-ldo-merkle/abi/*',
           'deployments/20210811-ldo-merkle/output/*',
           'deployments/20210812-lido-relayer/abi/*',
           'deployments/20210812-lido-relayer/output/*',
           'deployments/20210812-wsteth-rate-provider/abi/*',
           'deployments/20210812-wsteth-rate-provider/output/*',
           'deployments/20210907-investment-pool/abi/*',
           'deployments/20210907-investment-pool/output/*',
           'deployments/20210913-bal-arbitrum-merkle/abi/*',
           'deployments/20210913-bal-arbitrum-merkle/output/*',
           'deployments/20210928-mcb-arbitrum-merkle/abi/*',
           'deployments/20210928-mcb-arbitrum-merkle/output/*',
           'deployments/20211012-merkle-orchard/abi/*',
           'deployments/20211012-merkle-orchard/output/*',
           'deployments/20211202-no-protocol-fee-lbp/abi/*',
           'deployments/20211202-no-protocol-fee-lbp/output/*',
           'deployments/20211203-batch-relayer/abi/*',
           'deployments/20211203-batch-relayer/output/*',
           'deployments/20211208-aave-linear-pool/abi/*',
           'deployments/20211208-aave-linear-pool/output/*',
           'deployments/20211208-stable-phantom-pool/abi/*',
           'deployments/20211208-stable-phantom-pool/output/*',
           'deployments/20220304-erc4626-linear-pool/abi/*',
           'deployments/20220304-erc4626-linear-pool/output/*',
           'deployments/20220318-batch-relayer-v2/abi/*',
           'deployments/20220318-batch-relayer-v2/output/*',
           'deployments/20220325-authorizer-adaptor/abi/*',
           'deployments/20220325-authorizer-adaptor/output/*',
           'deployments/20220325-bal-token-holder-factory/abi/*',
           'deployments/20220325-bal-token-holder-factory/output/*',
           'deployments/20220325-balancer-token-admin/abi/*',
           'deployments/20220325-balancer-token-admin/output/*',
           'deployments/20220325-gauge-adder/abi/*',
           'deployments/20220325-gauge-adder/output/*',
           'deployments/20220325-gauge-controller/abi/*',
           'deployments/20220325-gauge-controller/output/*',
           'deployments/20220325-mainnet-gauge-factory/abi/*',
           'deployments/20220325-mainnet-gauge-factory/output/*',
           'deployments/20220325-single-recipient-gauge-factory/abi/*',
           'deployments/20220325-single-recipient-gauge-factory/output/*',
           'deployments/20220325-test-balancer-token/abi/*',
           'deployments/20220325-test-balancer-token/output/*',
           'deployments/20220325-ve-delegation/abi/*',
           'deployments/20220325-ve-delegation/output/*',
           'deployments/20220325-veBAL-deployment-coordinator/abi/*',
           'deployments/20220325-veBAL-deployment-coordinator/output/*',
           'deployments/20220413-arbitrum-root-gauge-factory/abi/*',
           'deployments/20220413-arbitrum-root-gauge-factory/output/*',
           'deployments/20220413-child-chain-gauge-factory/abi/*',
           'deployments/20220413-child-chain-gauge-factory/output/*',
           'deployments/20220413-polygon-root-gauge-factory/abi/*',
           'deployments/20220413-polygon-root-gauge-factory/output/*',
           'deployments/20220415-veBAL-L2-gauge-setup-coordinator/abi/*',
           'deployments/20220415-veBAL-L2-gauge-setup-coordinator/output/*',
           'deployments/20220418-veBAL-gauge-fix-coordinator/abi/*',
           'deployments/20220418-veBAL-gauge-fix-coordinator/output/*',
           'deployments/20220420-fee-distributor/abi/*',
           'deployments/20220420-fee-distributor/output/*',
           'deployments/20220420-smart-wallet-checker/abi/*',
           'deployments/20220420-smart-wallet-checker/output/*',
           'deployments/20220421-smart-wallet-checker-coordinator/abi/*',
           'deployments/20220421-smart-wallet-checker-coordinator/output/*',
           'deployments/20220422-distribution-scheduler/abi/*',
           'deployments/20220422-distribution-scheduler/output/*',
           'deployments/20220609-stable-pool-v2/abi/*',
           'deployments/20220609-stable-pool-v2/output/*',
           'deployments/20220725-protocol-fee-percentages-provider/abi/*',
           'deployments/20220725-protocol-fee-percentages-provider/output/*',
           'deployments/20220817-aave-rebalanced-linear-pool/abi/*',
           'deployments/20220817-aave-rebalanced-linear-pool/output/*',
           'deployments/20220906-composable-stable-pool/abi/*',
           'deployments/20220906-composable-stable-pool/output/*',
           'deployments/20220908-weighted-pool-v2/abi/*',
           'deployments/20220908-weighted-pool-v2/output/*',
           'deployments/20221021-managed-pool/abi/*',
           'deployments/20221021-managed-pool/output/*',
           'deployments/20230206-composable-stable-pool-v3/abi/*',
           'deployments/20230206-composable-stable-pool-v3/output/*',
           'deployments/20230206-erc4626-linear-pool-v3/abi/*',
           'deployments/20230206-erc4626-linear-pool-v3/output/*',
           'deployments/20230206-weighted-pool-v3/abi/*',
           'deployments/20230206-weighted-pool-v3/output/*']}

install_requires = \
['Cython==0.29.24',
 'cytoolz==0.11.2',
 'eth-abi==2.1.1',
 'gql==2.0.0',
 'jstyleson==0.0.2',
 'multicaller>=0.0.0a10',
 'requests==2.25.1',
 'web3==5.19.0']

setup_kwargs = {
    'name': 'balpy',
    'version': '0.0.0a81',
    'description': 'Balancer V2 Python API',
    'long_description': None,
    'author': 'Greg',
    'author_email': 'greg@orb.cl',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from decimal import *
from balancerv2cad.WeightedMath import WeightedMath


getcontext().prec = 28
MAX_RELATIVE_ERROR = Decimal(1e-17)

def expectEqualWithError(result: Decimal, expected: Decimal):
    if result <= expected + MAX_RELATIVE_ERROR and result >= expected - MAX_RELATIVE_ERROR:
        return True
    return False

class TestWeightedMath:

	def test_calculate_invariant(weightedmath_test):
		#Test instance of Decimal
		normalized_weight = [Decimal(0.3), Decimal(0.2), Decimal(0.5)]
		balances = [Decimal(10), Decimal(12), Decimal(14)]
		result = WeightedMath.calculate_invariant(normalized_weight, balances)
		assert isinstance(result, Decimal)

	def test_calculate_invariant_zero_invariant(weightedmath_test):
		#Tests zero Invariant
		result = WeightedMath.calculate_invariant([Decimal(1)], [Decimal(0)])
		assert result == Decimal(0)

	def test_calculate_invariant_two_tokens(weightedmath_test):
		#Test two tokens
		normalized_weight = [Decimal(0.3), Decimal(0.7)]
		balances = [Decimal(10), Decimal(12)]
		result = WeightedMath.calculate_invariant(normalized_weight, balances)
		expected = Decimal(11.361269771988886911)
		assert expectEqualWithError(result, expected)

	def test_calculate_invariant_three_tokens(weightedmath_test):
		#Test three tokens
		normalized_weight = [Decimal(0.3), Decimal(0.2), Decimal(0.5)]
		balances = [Decimal(10), Decimal(12), Decimal(14)]
		result = WeightedMath.calculate_invariant(normalized_weight, balances)
		expected = Decimal(12.271573899486561654)
		assert expectEqualWithError(result, expected)

	def test_out_given_in_single_swap(weightedmath_test):
		#Test instance of Decimal
		token_balance_in = Decimal(100)
		token_weight_in = Decimal(50)
		token_balance_out = Decimal(100)
		token_weight_out = Decimal(40)
		token_amount_out = Decimal(15)
		result = WeightedMath.calc_out_given_in(token_balance_in, token_weight_in, token_balance_out, token_weight_out, token_amount_out)
		assert isinstance(result, Decimal)

		#Test out given in 
		expected = Decimal(1602931431298673722)/Decimal(1e17)
		assert expectEqualWithError(result, expected)

	def test_in_given_out_single_swap(weightedmath_test):
		#Test instance of Decimal
		balance_in = Decimal(100)
		weight_in = Decimal(50)
		balance_out = Decimal(100)
		weight_out = Decimal(40)
		amount_out = Decimal(15)
		result = WeightedMath.calc_in_given_out(balance_in, weight_in, balance_out, weight_out, amount_out)
		assert isinstance(result, Decimal)
		
		expected = Decimal(1388456294146839515)/Decimal(1e17)
		assert expectEqualWithError(result, expected)
	
	def test_out_given_in_extreme_amounts(weightedmath_test):
		# Test instance of Decimal

		token_balance_in = Decimal(100)
		token_weight_in = Decimal(50)
		token_balance_out = Decimal(100)
		token_weight_out = Decimal(40)
		token_amount_out = Decimal(0.00000000001)
		result = WeightedMath.calc_out_given_in(token_balance_in, token_weight_in, token_balance_out, token_weight_out, token_amount_out)

		assert isinstance(result, Decimal)

		# Test out given in 
		# min amount in

		expected = Decimal(12500000)/Decimal(1e18)
		assert expectEqualWithError(result, expected)
	def test_in_given_out_extreme_amounts(weightedmath_test):
		#Test instance of Decimal
		token_balance_in = Decimal(100)
		token_weight_in = Decimal(50)
		token_balance_out = Decimal(100)
		token_weight_out = Decimal(40)
		token_amount_out = Decimal(0.00000000001)
		result = WeightedMath.calc_in_given_out(token_balance_in, token_weight_in, token_balance_out, token_weight_out, token_amount_out)

		assert isinstance(result, Decimal)

		# Test In given in 
		# min amount out
		expected = Decimal(8000000)/Decimal(1e18)
		assert expectEqualWithError(result, expected)

	def test_out_given_in_extreme_weights(weightedmath_test):
		# Max weights relation
		# Weight relation = 130.07
		token_balance_in = Decimal(100)
		token_weight_in = Decimal(130.7)
		token_balance_out = Decimal(100)
		token_weight_out = Decimal(1)
		token_amount_out = Decimal(15)
		result = WeightedMath.calc_out_given_in(token_balance_in, token_weight_in, token_balance_out, token_weight_out, token_amount_out)

		assert isinstance(result, Decimal)

		#Test out given in 
		expected = Decimal(9999999883374836452)/Decimal(1e17)
		assert expectEqualWithError(result, expected)

	def test_in_given_out_extreme_weights(weightedmath_test):
		# Min weights relation
		# Weight relation = 0.00769
		token_balance_in = Decimal(100)
		token_weight_in = Decimal(0.00769)
		token_balance_out = Decimal(100)
		token_weight_out = Decimal(1)
		token_amount_out = Decimal(15)
		result = WeightedMath.calc_out_given_in(token_balance_in, token_weight_in, token_balance_out, token_weight_out, token_amount_out)

		assert isinstance(result, Decimal)

		#Test out given in 
		expected = Decimal(0.107419197916188066)
		assert expectEqualWithError(result, expected)


	
	def test_calc_due_token_protocol_swap_fee_amount_two_tokens(weightedmath_test):
		# Returns protocl swap fees
		normalized_weights = [Decimal(0.3), Decimal(0.7)]
		balances = [Decimal(10), Decimal(11)]
		last_invariant = Decimal(10)
		token_index = 1
		current_invariant = Decimal(10.689930449163329926)
		protocol_swap_fee_percentage = Decimal(0.1)
		result = WeightedMath.calc_due_token_protocol_swap_fee_amount(balances[token_index], normalized_weights[token_index], last_invariant, current_invariant, protocol_swap_fee_percentage)
		assert isinstance(result, Decimal)
		expected = Decimal(0.099999999999999999933)
		assert expectEqualWithError(result,expected)

		# With large accumulated fees, caps the invariant growth
		normalized_weights = [Decimal(0.3), Decimal(0.7)]
		balances = [Decimal(10), Decimal(11)]
		token_index = 1
		current_invariant = Decimal(10.689930449163329926)
		last_invariant = current_invariant/Decimal(2)
		protocol_swap_fee_percentage = Decimal(0.1)
		result = WeightedMath.calc_due_token_protocol_swap_fee_amount(balances[token_index], normalized_weights[token_index], last_invariant, current_invariant, protocol_swap_fee_percentage)
		
		assert isinstance(result, Decimal)
		expected = Decimal(0.439148057504926669190)
		assert expectEqualWithError(result,expected)
	
	def test_calc_due_token_protocol_swap_fee_amount_three_tokens(weightedmath_test):
		normalized_weights = [Decimal(0.3), Decimal(0.2), Decimal(0.5)]
		balances = [Decimal(10), Decimal(11), Decimal(12)]
		last_invariant = Decimal(10)

		token_index = 2
		current_invariant = Decimal(11.1652682095187556376)
		
		protocol_swap_fee_percentage = Decimal(0.1)
		result = WeightedMath.calc_due_token_protocol_swap_fee_amount(balances[token_index], normalized_weights[token_index], last_invariant, current_invariant, protocol_swap_fee_percentage)
		
		assert isinstance(result, Decimal)
		expected = Decimal(0.23740649734383223657)
		assert expectEqualWithError(result,expected)



	def test_calc_bpt_out_given_exact_tokens_in(weightedmath_test):
		#Test instance of Decimal
		balances = [Decimal(10), Decimal(12), Decimal(14)]
		normalized_weights = [Decimal(10), Decimal(12), Decimal(14)]
		amounts_in = [Decimal(10), Decimal(12), Decimal(14)]
		bpt_total_supply = Decimal(1)
		swap_fee = Decimal(1)
		result = WeightedMath.calc_bpt_out_given_exact_tokens_in(balances, normalized_weights, amounts_in, bpt_total_supply, swap_fee)
		assert isinstance(result, Decimal)

	def test_calc_token_in_given_exact_bpt_out(weightedmath_test):
		#Test instance of Decimal
		balance = Decimal(1)
		normalized_weight = Decimal(10)
		bpt_amount_out = Decimal(10)
		bpt_total_supply = Decimal(10)
		swap_fee = Decimal(10)
		result = WeightedMath.calc_token_in_given_exact_bpt_out(balance, normalized_weight, bpt_amount_out, bpt_total_supply, swap_fee)
		assert isinstance(result, Decimal)

	def test_calc_bpt_in_given_exact_tokens_out(weightedmath_test):
		#Test instance of Decimal
		balances = [Decimal(10), Decimal(12), Decimal(14)]
		normalized_weights = [Decimal(10), Decimal(12), Decimal(14)]
		bpt_amount_out = [Decimal(10), Decimal(12), Decimal(14)]
		bpt_total_supply = Decimal(1)
		swap_fee = Decimal(1)
		result = WeightedMath.calc_bpt_in_given_exact_tokens_out(balances, normalized_weights, bpt_amount_out, bpt_total_supply, swap_fee)
		assert isinstance(result, Decimal)

	def test_calc_token_out_given_exact_bpt_in(weightedmath_test):
		#Test instance of Decimal
		balance = Decimal(10)
		normalized_weight = Decimal(10)
		bpt_amount_in = Decimal(2)
		bpt_total_supply = Decimal(10)
		swap_fee = Decimal(0)
		result = WeightedMath.calc_token_out_given_exact_bpt_in(balance, normalized_weight, bpt_amount_in, bpt_total_supply, swap_fee)
		assert isinstance(result, Decimal)

	def test_calc_tokens_out_given_exact_bpt_in(weightedmath_test):
		#Test instance of Decimal
		balance = Decimal(10)
		normalized_weight = Decimal(1)
		bpt_amount_in = Decimal(2)
		bpt_total_supply = Decimal(1)
		swap_fee = Decimal(1)
		result = WeightedMath.calc_token_out_given_exact_bpt_in(balance, normalized_weight, bpt_amount_in, bpt_total_supply, swap_fee )
		assert isinstance(result, Decimal)

	



"""
Define functions as project "scripts" and run them via the
pyproject.toml
"""
import subprocess as sp
import sys

import balancerv2cad as ks
# from balancerv2cad import PKG_NAME, BASE_DIR


def stub_gen() -> None:
    """
    Generate all function stubs
    for the package
    """
    out_file = f"src/{ks.__package__}-stubs"
    path = str(ks.BASE_DIR / 'src' / f'{ks.__package__}')
    try:
        sp.run(f'stubgen -p {ks.__package__} -o {out_file}',
               check=True,
               shell=True)
        sp.run(f'mypy {path}', check=True, shell=True)
    except sp.CalledProcessError as error:
        print(f'{error}')
        sys.exit(1)


def run_analyzer() -> None:
    """
    Run the mypy static type checking analyzer
    """
    path = str(ks.BASE_DIR / 'src' / f'{ks.__package__}')
    try:
        sp.run(f'mypy {path}', check=True, shell=True)
    except sp.CalledProcessError as error:
        print(f'{error}')
        sys.exit(1)


def run_tests():
    """
    An example function to be used as a python poetry
    script. This can be run issueing the command
    'poetry run tests'
    These functions can be as complicated or as simple as you like
    """
    try:
        sp.run('pytest --capture=tee-sys', check=True, shell=True)
    except sp.CalledProcessError as error:
        print(f"{error}")
        sys.exit(1)



"""
Showcasing two different ways to properly test your code

1. By importing the balancerv2cad package in directly.
2. By passing in a function fixture defined in conftest.py
"""
import balancerv2cad as ks



def test_pkgname_using_fixture(version_test):
    """
    testing pkgname from passing the module
    in as a fixture
    """
    assert version_test.__package__ == 'balancerv2cad'



# basics
import json
import math
import sys
import time

# thegraph queries
from gql import gql, Client
from gql.transport.requests import RequestsHTTPTransport

# for customized endpoints
import requests

class TheGraph(object):
	client = None;
	
	"""
	A starting point for querying pool data from the Balancer Subgraph.
		At time of writing, this package does not cover all types of queries
		possible to the Balancer V2 Subgraph. It does, however, allow users 
		to query pool data, among other things. Types of queries will ultimately
		grow to include all possible queries to the Balancer Subgraph.

		For more information on the Subgraph, please go to:
		https://thegraph.com/legacy-explorer/subgraph/balancer-labs/balancer-v2
	"""

	def __init__(self, network="mainnet", customUrl=None, usingJsonEndpoint=False):
		super(TheGraph, self).__init__()
		self.network = network;
		self.initBalV2Graph(customUrl=customUrl, usingJsonEndpoint=usingJsonEndpoint);

	def printJson(self, curr_dict):
		print(json.dumps(curr_dict, indent=4))

	def callCustomEndpoint(self, query):

		query = query.replace("\n"," ");
		query = query.replace("\t","");
		queryDict = {"query":query};
		serializedData = json.dumps(queryDict);
		headers = {"Content-Type":"application/json"};
		r = requests.post(self.graphUrl, data=serializedData, headers=headers);
		response = r.json();
		return(response["data"])

	def assertInit(self):
		if self.client is None:
			print()
			print("[ERROR] Subgraph not initialized.");
			print("Call \"initBalV2Graph(network)\" before calling this function.")
			print()
			return(None);

	def initBalV2Graph(self, customUrl, usingJsonEndpoint, verbose=False):
		if not customUrl is None and usingJsonEndpoint:
			self.client = "CUSTOM"
			self.graphUrl = customUrl;
			return(True);

		network_string = "";
		if not self.network == "mainnet":
			network_string = "-" + self.network;

		if verbose:
			print("Balancer V2 Subgraph initializing on:", self.network, "...")

		graphUrl = "https://api.thegraph.com/subgraphs/name/balancer-labs/balancer" + network_string + "-v2";
		if not customUrl is None and not usingJsonEndpoint:
			graphUrl = customUrl;

		balancer_transport=RequestsHTTPTransport(
		    url=graphUrl,
		    verify=True,
		    retries=3
		)
		self.client = Client(transport=balancer_transport)
		
		if verbose:
			print("Successfully initialized on network:", self.network);

	def getPoolTokens(self, pool_id, verbose=False):

		self.assertInit();

		if verbose:
			print("Querying tokens for pool with ID:", pool_id)

		pool_token_query = '''
		query {{
		  poolTokens(first: 8, where: {{ poolId: "{pool_id}" }}) {{
		    id
			poolId {{
				id
			}}
			symbol
			name
			decimals
			address
			balance
			weight
		  }}
		}}
		''';
		formatted_query_string = pool_token_query.format(pool_id=pool_id)
		if self.client == "CUSTOM":
			response = self.callCustomEndpoint(formatted_query_string);
		else:
			response = self.client.execute(gql(formatted_query_string))
		if verbose:
			print("Got pool tokens.")
		return(response["poolTokens"])

	def getNumPools(self, verbose=False):

		self.assertInit();
		if verbose:
			print("Querying number of pools...")

		# get number of balancer pools on v2
		balancers_query = '''
		query {
			balancers(first: 5) {
		    id
		    poolCount
		  }
		}
		'''
		if self.client == "CUSTOM":
			response = self.callCustomEndpoint(balancers_query);
		else:
			response = self.client.execute(gql(balancers_query));

		if verbose:
			print("Got response from the Subgraph")

		for balancer in response["balancers"]:
			if balancer["id"] == "2":
				num_pools = balancer["poolCount"]
				return(num_pools)
		return None;

	def getPools(self, batch_size, skips, verbose=False):

		self.assertInit();
		if verbose:
			print("Querying pools #", skips, "through #", skips + batch_size, "...")

		query_string = '''
			query {{
			  pools(first: {first}, skip: {skip}) {{
			    id
			    address
			    poolType
			    strategyType
			    swapFee
			  }}
			}}
			'''
		formatted_query_string = query_string.format(first=batch_size, skip=skips)
		if self.client == "CUSTOM":
			response = self.callCustomEndpoint(formatted_query_string);
		else:
			response = self.client.execute(gql(formatted_query_string))

		if verbose:
			print("Got pools.")
		return(response)

	def getV2Pools(self, batch_size, verbose=False):

		if self.client is None:
			self.initBalV2Graph(verbose=verbose);
		
		num_pools = self.getNumPools(verbose=verbose);
		num_calls = math.ceil(num_pools/batch_size)

		if verbose:
			print("Querying",num_pools, "pools...");

		# query all pools by batch to save time
		pool_tokens = {};
		for i in range(num_calls):
			response = self.getPools(batch_size, batch_size*i, verbose)
			
			for pool in response["pools"]:
				curr_id = pool["id"]
				curr_pool_token_data = self.getPoolTokens(curr_id, verbose=verbose);
				pool_data = {};
				pool_data["tokens"] = curr_pool_token_data;
				pool_data["poolType"] = pool["poolType"];
				pool_data["swapFee"] = pool["swapFee"];
				pool_tokens[curr_id] = pool_data;
		return(pool_tokens)

	def getV2PoolIDs(self, batch_size, verbose=False):

		if self.client is None:
			self.initBalV2Graph(verbose=verbose);

		num_pools = self.getNumPools(verbose=verbose);
		num_calls = math.ceil(num_pools/batch_size)

		if verbose:
			print("Querying",num_pools, "pools...");

		# query all pools by batch to save time
		poolIdsByType = {};
		for i in range(num_calls):
			response = self.getPools(batch_size, batch_size*i, verbose)
			for pool in response["pools"]:
				if pool["poolType"] not in poolIdsByType.keys():
					poolIdsByType[pool["poolType"]] = [];
				poolIdsByType[pool["poolType"]].append(pool["id"])
		header = {};
		header["stamp"] = time.time();
		poolCount = 0
		for t in poolIdsByType.keys():
			poolCount += len(poolIdsByType[t])
		header["numPools"] = poolCount;

		data = {};
		data["header"] = header;
		data["pools"] = poolIdsByType
		return(data)

	def getPoolBptPriceEstimate(self, poolId, verbose=False):

		self.assertInit();
		if verbose:
			print("Getting data for pool", poolId, "from the subgraph...")

		query_string = '''
			query {{
				pools(where:{{id: "{poolId}"}}) {{
					totalShares
					totalLiquidity
				}}
			}}
			'''
		formatted_query_string = query_string.format(poolId=poolId);
		response = self.client.execute(gql(formatted_query_string));

		pool = response["pools"][0]
		pricePerBpt = float(pool["totalLiquidity"])/float(pool["totalShares"])

		if verbose:
			print("Got price data:", pricePerBpt)
		return(pricePerBpt)

	def getPoolsAndTokens(self, batch_size, skips, verbose=False):
		self.assertInit();
		query_string = '''
			query {{
			  pools(first: {first}, skip: {skip}) {{
			    id
			    tokens {{
			      token
			      {{
			        id
			      }}
			    }}
			  }}
			}}
		'''
		formatted_query_string = query_string.format(first=batch_size, skip=skips)

		response = self.client.execute(gql(formatted_query_string));
		if self.client == "CUSTOM":
			response = self.callCustomEndpoint(formatted_query_string);
		else:
			response = self.client.execute(gql(formatted_query_string))
		return(response)

def main():
	
	batch_size = 30;
	print();

	if len(sys.argv) < 2:
		print("Usage: python", sys.argv[0], "<network>");
		print("No network given; defaulting to mainnet Ethereum")
		network = "mainnet"
	else:
		network = sys.argv[1];
	
	networks = ["mainnet", "kovan", "polygon", "arbitrum"];

	if not network in networks:
		print("Network", network, "is not supported!");
		print("Supported networks are:");
		for n in networks:
			print("\t" + n);
		print("Quitting")
		quit();

	verbose = True;

	graph = TheGraph(network)
	# pools = graph.getNumPools(verbose=verbose)
	pools = graph.getV2Pools(batch_size, verbose=verbose)
	graph.printJson(pools)

if __name__ == '__main__':
	main();
